#!/usr/bin/env ruby
# encoding: utf-8

module Coopy
  class Index 
    
    def initialize(flags)
      @items = {}
      @cols = Array.new
      @keys = Array.new
      @top_freq = 0
      @height = 0
      @hdr = 0
      @ignore_whitespace = false
      @ignore_case = false
      if flags != nil 
        @ignore_whitespace = flags.ignore_whitespace
        @ignore_case = flags.ignore_case
      end
    end
    
    attr_accessor :items
    attr_accessor :keys
    attr_accessor :top_freq
    attr_accessor :height
    
    protected
    
    attr_accessor :cols
    attr_accessor :v
    attr_accessor :indexed_table
    attr_accessor :hdr
    attr_accessor :ignore_whitespace
    attr_accessor :ignore_case
    
    public
    
    def add_column(i)
      @cols.push(i)
    end
    
    def index_table(t,hdr)
      @indexed_table = t
      @hdr = hdr
      @keys[t.get_height - 1] = nil if @keys.length != t.get_height && t.get_height > 0
      begin
        _g1 = 0
        _g = t.get_height
        while(_g1 < _g) 
          i = _g1
          _g1+=1
          key = @keys[i]
          if key == nil 
            key = self.to_key(t,i)
            @keys[i] = key
          end
          item = @items[key]
          if item == nil 
            item = ::Coopy::IndexItem.new
            @items[key] = item
          end
          ct = nil
          begin
            item.lst = Array.new if item.lst == nil
            item.lst.push(i)
            ct = item.lst.length
          end
          @top_freq = ct if ct > @top_freq
        end
      end
      @height = t.get_height
    end
    
    def to_key(t,i)
      wide = nil
      if i < @hdr 
        wide = "_"
      else 
        wide = ""
      end
      @v = t.get_cell_view if @v == nil
      begin
        _g1 = 0
        _g = @cols.length
        while(_g1 < _g) 
          k = _g1
          _g1+=1
          d = t.get_cell(@cols[k],i)
          txt = @v.to_s(d)
          txt = txt.strip if @ignore_whitespace
          txt = txt.downcase if @ignore_case
          wide += " // " if k > 0
          next if txt == nil || txt == "" || txt == "null" || txt == "undefined"
          wide += txt
        end
      end
      wide
    end
    
    def to_key_by_content(row)
      wide = nil
      if row.is_preamble 
        wide = "_"
      else 
        wide = ""
      end
      begin
        _g1 = 0
        _g = @cols.length
        while(_g1 < _g) 
          k = _g1
          _g1+=1
          txt = row.get_row_string(@cols[k])
          txt = txt.strip if @ignore_whitespace
          txt = txt.downcase if @ignore_case
          wide += " // " if k > 0
          next if txt == nil || txt == "" || txt == "null" || txt == "undefined"
          wide += txt
        end
      end
      wide
    end
    
    def get_table 
      @indexed_table
    end
    
  haxe_me ["coopy", "Index"]
  end

end
